@@ -179,6 +179,18 @@ def _make_arg_parser():
               "validate existing packages"),
         default=False
     )
+    ap.add_argument(
+        '--no-validate-target',
+        action="store_true",
+        help="Skip validation of files already present in target-directory",
+        default=False,
+    )
+    ap.add_argument(
+        '--minimum-free-space',
+        help=("Threshold for free diskspace. Given in megabytes."),
+        type=int,
+        default=1000,
+    )
     return ap
 
 
@@ -214,6 +226,8 @@ def _parse_and_format_args():
     """
     parser = _make_arg_parser()
     args = parser.parse_args()
+    # parse arguments without setting defaults
+    given_args, _ = parser._parse_known_args(sys.argv[1:], argparse.Namespace())
 
     _init_logger(args.verbose)
     logger.debug('sys.argv: %s', sys.argv)
@@ -223,16 +237,6 @@ def _parse_and_format_args():
         print(__version__)
         sys.exit(1)
 
-    for required in ('target_directory', 'platform', 'upstream_channel'):
-        if not getattr(args, required):
-            raise ValueError("Missing command line argument: %s", required)
-
-    if args.pdb:
-        # set the pdb_hook as the except hook for all exceptions
-        def pdb_hook(exctype, value, traceback):
-            pdb.post_mortem(traceback)
-        sys.excepthook = pdb_hook
-
     config_dict = {}
     if args.config:
         logger.info("Loading config from %s", args.config)
@@ -240,9 +244,32 @@ def _parse_and_format_args():
             config_dict = yaml.load(f)
         logger.info("config: %s", config_dict)
 
+        # use values from config file unless explicitly given on command line
+        for a in parser._actions:
+            if (
+                # value exists in config file
+                (a.dest in config_dict) and
+                # ignore values that can only be given on command line
+                (a.dest not in {'config', 'verbose', 'version'}) and
+                # only use config file value if the value was not explicitly given on command line
+                (a.dest not in given_args)
+            ):
+                logger.info("Using %s value from config file", a.dest)
+                setattr(args, a.dest, config_dict.get(a.dest))
+
     blacklist = config_dict.get('blacklist')
     whitelist = config_dict.get('whitelist')
 
+    for required in ('target_directory', 'platform', 'upstream_channel'):
+        if (not getattr(args, required)):
+            raise ValueError("Missing command line argument: %s", required)
+
+    if args.pdb:
+        # set the pdb_hook as the except hook for all exceptions
+        def pdb_hook(exctype, value, traceback):
+            pdb.post_mortem(traceback)
+        sys.excepthook = pdb_hook
+
     return {
         'upstream_channel': args.upstream_channel,
         'target_directory': args.target_directory,
@@ -252,6 +279,8 @@ def _parse_and_format_args():
         'blacklist': blacklist,
         'whitelist': whitelist,
         'dry_run': args.dry_run,
+        'no_validate_target': args.no_validate_target,
+        'minimum_free_space': args.minimum_free_space,
     }
 
 
@@ -307,24 +336,28 @@ def _validate(filename, md5=None, size=None):
     reason : str
         The reason why the package is being removed
     """
-    try:
-        t = tarfile.open(filename)
-        t.extractfile('info/index.json').read().decode('utf-8')
-    except (tarfile.TarError, EOFError):
-        logger.info("Validation failed because conda package is corrupted.",
-                    exc_info=True)
-        return _remove_package(filename, reason="Tarfile read failure")
-    if size:
-        if os.stat(filename).st_size != size:
-            return _remove_package(filename, reason="Failed size test")
     if md5:
         calc = hashlib.md5(open(filename, 'rb').read()).hexdigest()
-        if calc != md5:
+        if calc == md5:
+            # If the MD5 matches, skip the other checks
+            return filename, None
+        else:
             return _remove_package(
                 filename,
                 reason="Failed md5 validation. Expected: %s. Computed: %s"
                 % (calc, md5))
 
+    if size and size != os.stat(filename).st_size:
+        return _remove_package(filename, reason="Failed size test")
+
+    try:
+        with tarfile.open(filename) as t:
+            t.extractfile('info/index.json').read().decode('utf-8')
+    except (tarfile.TarError, EOFError):
+        logger.info("Validation failed because conda package is corrupted.",
+                    exc_info=True)
+        return _remove_package(filename, reason="Tarfile read failure")
+
     return filename, None
 
 
@@ -369,7 +402,13 @@ def _download(url, target_directory):
         The url to download
     target_directory : str
         The path to a directory where `url` should be downloaded
+
+    Returns
+    -------
+    file_size: int
+        The size in bytes of the file that was downloaded
     """
+    file_size = 0
     chunk_size = 1024  # 1KB chunks
     logger.info("download_url=%s", url)
     # create a temporary file
@@ -380,6 +419,8 @@ def _download(url, target_directory):
         ret = requests.get(url, stream=True)
         for data in ret.iter_content(chunk_size):
             tf.write(data)
+        file_size = os.path.getsize(download_filename)
+    return file_size
 
 
 def _list_conda_packages(local_dir):
@@ -438,12 +479,12 @@ def _validate_packages(package_repodata, package_directory, num_threads=1):
                           package_directory)
                          for num, package in enumerate(sorted(local_packages))]
 
-    if num_threads is 1 or num_threads is None:
+    if num_threads == 1 or num_threads is None:
         # Do serial package validation (Takes a long time for large repos)
         validation_results = map(_validate_or_remove_package,
                                  val_func_arg_list)
     else:
-        if num_threads is 0:
+        if num_threads == 0:
             num_threads = os.cpu_count()
             logger.debug('num_threads=0 so it will be replaced by all available '
                          'cores: %s' % num_threads)
@@ -496,7 +537,7 @@ def _validate_or_remove_package(args):
         return _remove_package(package_path, reason=reason)
     # validate the integrity of the package, the size of the package and
     # its hashes
-    logger.info('Validating {:4d} of {:4d}: {}.'.format(num, num_packages,
+    logger.info('Validating {:4d} of {:4d}: {}.'.format(num + 1, num_packages,
                                                         package))
     package_path = os.path.join(package_directory, package)
     return _validate(package_path,
@@ -505,7 +546,8 @@ def _validate_or_remove_package(args):
 
 
 def main(upstream_channel, target_directory, temp_directory, platform,
-         blacklist=None, whitelist=None, num_threads=1, dry_run=False):
+         blacklist=None, whitelist=None, num_threads=1, dry_run=False,
+         no_validate_target=False, minimum_free_space=0):
     """
 
     Parameters
@@ -542,6 +584,11 @@ def main(upstream_channel, target_directory, temp_directory, platform,
         Defaults to False.
         If True, skip validation and exit after determining what needs to be
         downloaded and what needs to be removed.
+    no_validate_target : bool, optional
+        Defaults to False.
+        If True, skip validation of files already present in target_directory.
+    minimum_free_space : int, optional
+        Stop downloading when free space target_directory or temp_directory reach this threshold.
 
     Returns
     -------
@@ -653,7 +700,7 @@ def main(upstream_channel, target_directory, temp_directory, platform,
     # construct the desired package repodata
     desired_repodata = {pkgname: packages[pkgname]
                         for pkgname in possible_packages_to_mirror}
-    if not dry_run:
+    if not (dry_run or no_validate_target):
         # Only validate if we're not doing a dry-run
         validation_results = _validate_packages(desired_repodata, local_directory, num_threads)
         summary['validating-existing'].update(validation_results)
@@ -674,6 +721,8 @@ def main(upstream_channel, target_directory, temp_directory, platform,
     # b. validate contents of temp file
     # c. move to local repo
     # mirror all new packages
+    total_bytes = 0
+    minimum_free_space_kb = (minimum_free_space * 1024 * 1024)
     download_url, channel = _maybe_split_channel(upstream_channel)
     with tempfile.TemporaryDirectory(dir=temp_directory) as download_dir:
         logger.info('downloading to the tempdir %s', download_dir)
@@ -682,8 +731,27 @@ def main(upstream_channel, target_directory, temp_directory, platform,
                 channel=channel,
                 platform=platform,
                 file_name=package_name)
-            _download(url, download_dir)
-            summary['downloaded'].add((url, download_dir))
+            try:
+                # make sure we have enough free disk space in the temp folder to meet threshold
+                if shutil.disk_usage(download_dir).free < minimum_free_space_kb:
+                    logger.error('Disk space below threshold in %s. Aborting download.',
+                                 download_dir)
+                    break
+
+                # download package
+                total_bytes += _download(url, download_dir)
+
+                # make sure we have enough free disk space in the target folder to meet threshold
+                # while also being able to fit the packages we have already downloaded
+                if (shutil.disk_usage(local_directory).free - total_bytes) < minimum_free_space_kb:
+                    logger.error('Disk space below threshold in %s. Aborting download',
+                                 local_directory)
+                    break
+
+                summary['downloaded'].add((url, download_dir))
+            except Exception as ex:
+                logger.exception('Unexpected error: %s. Aborting download.', ex)
+                break
 
         # validate all packages in the download directory
         validation_results = _validate_packages(packages, download_dir,
